#include <string.h>
#include <stdio.h>
#include "fitsio.h"





int imcalet(char *filename, double val_min, double val_max, double *etime)
{
	fitsfile *fptr; /* FITS file pointer */
	int status = 0; /* CFITSIO status value MUST be initialized to zero! */
	int hdutype, naxis, ii;
	long naxes[2], totpix, fpixel[2], rowlen;
	double *pix, maxval = -1.E33;
	double exposure;
	//,sum = 0., meanval = 0., minval = 1.E33, maxval = -1.E33;
	int count_max = 0;
	int count_min = 0;
	int count = 0;
	int sum;
	int histogram[4096];

	if ( !fits_open_file(&fptr, filename, READONLY, &status) ) {
		if (fits_get_hdu_type(fptr, &hdutype, &status) || 
		    hdutype != IMAGE_HDU) { 
			printf("Error: this program only works on images, not tables\n");
			return -1;
		}
		if ( fits_read_key(fptr, TDOUBLE, "EXPTIME", &exposure, NULL,
				   &status) ) {
			//PVERB(PERR, "getting %s: EXPTIME not found\n", file);
			return -1;
		}

		fits_get_img_dim(fptr, &naxis, &status);
		fits_get_img_size(fptr, 2, naxes, &status);

		if (status || naxis != 2) { 
			printf("Error: NAXIS = %d.  Only 2-D images are supported.\n", naxis);
			return -1;
		}

		/* memory for 1 row */
		pix = (double *) malloc(naxes[0] * sizeof(double));

		if (pix == NULL) {
			printf("Memory allocation error\n");
			return -1;
		}

		totpix = naxes[0] * naxes[1];
		fpixel[0] = 1; /* read starting with first pixel in
                                  each row */

		for (ii=0; ii < 4096; ii++) histogram[ii] = 0;
		/* process image one row at a time; increment row # in
                   each loop */
		for (fpixel[1] = 1; fpixel[1] <= naxes[1]; fpixel[1]++)  {  
			/* give starting pixel coordinate and number
                           of pixels to read */
			if (fits_read_pix(fptr, TDOUBLE, fpixel, naxes[0],
					  0, pix,0, &status))
				break;   /* jump out of loop on error */

			for (ii = 0; ii < naxes[0]; ii++) {
				histogram[((int) pix[ii]) & 0xfff]++;
				if ( pix[ii] > val_min &&
				     pix[ii] < val_max ) {
					count_min++;
				}else if (pix[ii] > val_max) {
					count_max++;
				}
				if (pix[ii] > maxval) {
					maxval = pix[ii];
					count = 1;
				} else if (pix[ii] == maxval) {
					count++;
				}

	   
			}
		}
      
		free(pix);
		fits_close_file(fptr, &status);
	}

	if (status)  {
		/* print any error message */
		fits_report_error(stderr, status);
	} else {
		#if 0
		printf("Statistics of %ld x %ld image (%ld pixels)\n",
		       naxes[0], naxes[1], totpix);
		printf("  maximum value = %g in %d pixels\n", maxval, count);
		printf("  %d %d\n", count_min, count_max);  
		ii = 4095;
		sum=0;
		while(ii > 0 && sum < 100) sum+=histogram[ii--];
		ii++;
		printf("Histogram: %d in %d pixels\n", ii+1, histogram[ii+1]);
		printf("Histogram: %d in %d pixels\n", ii, histogram[ii]);
		printf("Histogram: %d in %d pixels\n", ii-1, histogram[ii-1]);
		*etime = exposure * (val_min+val_max) / 
			((double) (ii) *2.0);
		printf("Exposure time (old %.1f): %.1f\n", exposure, *etime);
		#else
		sum = 0;
		for(ii=0;ii<4096;ii++) {
			sum += histogram[ii];
			printf("%4d %4d %4d\n", ii, histogram[ii], sum);
		}
		#endif

	}

	return status;
}


int main(int argc, char *argv[])
{
	double etime;

	return imcalet(argv[1], 3500.0, 4000.0, &etime);
}

